    .data
    .globl array
array:
    .space 20

    .text
    .globl array_fattoriale

fattoriale:
    # Abbiamo 3 cose da salvare, $fp, $ra e $a0
    # FP è l'unico espresso in relazione a $sp, infatti è positivo
    .eqv FP_1, 8
    .eqv RA_1, -4
    .eqv T0_1, -8

    # Riserva 12 byte, ovvero 3 slot a da 32-bit
    addiu $sp, -12
    # Salva il registro $fp e aggiornalo facendolo puntare alla posizione dove il
    # vecchio $fp è appena stato salvato
    sw $fp, FP_1($sp)
    addiu $fp, $sp, FP_1

    # Salva i callee-saved
    sw $ra, RA_1($fp)

    # Effettua il confronto con 0, se non corrisponde, salta al ramo else
    bne $zero, $a0, else

    # Restituisci il parametro i e salta alla fine della funzione
    li $v0, 1
    j endif
else:
    # Salva i caller-saved che ci servono ($t0, che contiene una copia di $a0
	# prima che venga modificato), passa a fattoriale (i - 1) e ripristina $t0
    move $t0, $a0
    addi $a0, $a0, -1
    sw $t0, T0_1($fp)
    jal fattoriale
    lw $t0, T0_1($fp)

    # return $v0 * $t0
    mul $v0, $v0, $t0

endif:
    # Ripristina i callee-saved, $fp e lo stack
    lw $ra, RA_1($fp)
    lw $fp, 0($fp)
    addiu $sp, 12

    j $ra

array_fattoriale:
    # Abbiamo 4 cose da salvare: il frame pointer, $ra, $s0 e $s1
    # FP è l'unico espresso in relazione a $sp, infatti è positivo
    .eqv FP_2, 12
    .eqv RA_2, -4
    .eqv S0_2, -8
    .eqv S1_2, -12

    # Riserva 16 byte, ovvero 4 slot da 32-bit
    addiu $sp, -16
    # Salva il registro $fp e aggiornalo facendolo puntare alla posizione dove il
    # vecchio $fp è appena stato salvato
    sw $fp, FP_2($sp)
    addiu $fp, $sp, FP_2

    # Salva i callee-saved
    sw $ra, RA_2($fp)
    sw $s0, S0_2($fp)
    sw $s1, S1_2($fp)

    # Inizializza il contatore i a 0 a carica l'indirizzo dell'array
    li $s0, 0
    la $s1, array

loop_head:
    # Controlla la condizione del ciclo negata, se si verifica la condizione,
    # esci dal ciclo
    li $t0, 5
    bge $s0, $t0, loop_end

    # Passa alla funzione fattoriale il valore di i come primo parametro
    move $a0, $s0
    jal fattoriale

    # Calcola l'indirizzo dove salvare il risultato
    # *($s1 + ($s0 << 2)) = $v0
    # o anche:
    # *($s1 + $s0 * 4) = $v0
    sll $t0, $s0, 2
    addu $t0, $s1, $t0
    sw $v0, ($t0)

    # Incrementa il contatore
    addiu $s0, $s0, 1

    # Effettua un'altra iterazione del ciclo
    j loop_head
loop_end:

    # Riprisinta i callee-saved, il $fp e lo stack
    lw $s0, S0_2($fp)
    lw $s1, S1_2($fp)
    lw $ra, RA_2($fp)
    lw $fp, 0($fp)
    addiu $sp, 16

    j $ra
